// pvm.cc 

#include "pvm.h"

#include <stdlib.h>
#include <iostream.h>
#include <strstream.h>

PVM::PVM(char* executable_name)
{ 
    assert((mytid=pvm_mytid())>=0,"couldn't create PVM object");
    need_vmdata=true;
    exname=new char[strlen(executable_name)+1];
    strcpy(exname,executable_name);
}

PVM::~PVM()
{ 
    pvm_exit();		// exit pvm
    delete exname;
}

void 
PVM::kill(char* msg)
{
    int buflen=strlen(msg)+30;
    char buf[buflen];
    ostrstream ost(buf,buflen);
    ost<<"Client error: "<<msg;
    die(buf);
}

int 
PVM::wait_next_mesg(int tag)
{
    int n;
    assert(n=pvm_recv(-1, tag)>=0,"next_mesg");
    return n;
}

int 
PVM::unpack_int()
{
    int data;
    assert(pvm_upkint(&data,1,1)>=0, "unpack_int");
    return data;
}

int*
PVM::unpack_intarray(int len)
{
    int* data;
    assert(pvm_upkint(data,len,1)>=0, "unpack_intarray");
    return data;
}

void
PVM::unpack_str(char *buf)
{
    assert(pvm_upkstr(buf)>=0, "unpack_str");
}

void 
PVM::init_send(int encoding)
{
    assert(pvm_initsend(encoding)>=0, "init_send");
}

void  
PVM::pack_int(int n)
{
    assert(pvm_pkint(&n, 1, 1)>=0, "pack_int"); 
}

void 
PVM::pack_str(char* s)
{
    assert(pvm_pkstr(s)>=0, "pack_str"); 
}

void 
PVM::send_to_parent(int tag)
{
    send_to_task(parent_taskid(), tag);
}

void 
PVM::send_to_task(int taskid, int tag)
{
    assert(pvm_send(taskid, tag)>=0, "send_to_task");
}

bool 
PVM::noparent()
{
    return parent_taskid()==PvmNoParent; 
}

int
PVM::parent_taskid() 
{ 
    int myparent=pvm_parent();
    assert(myparent >= 0  || myparent == PvmNoParent, "parent_taskid");
    return myparent;
}

void
PVM::spawn(int children[], int ntask, char* argv[])
{
    assert(pvm_spawn(exname, argv, PvmTaskDefault,
		  (char*)NULL, ntask, children) == ntask,
	   "spawn(childrenntask,argv) failed.");
}

void
PVM::spawn_onhost(int children[], char* hname, int ntask, char* argv[])
{
    assert(pvm_spawn(exname, argv, PvmTaskHost,
		  hname, ntask, children) == ntask,
	   "spawn(childrenntask,argv) failed.");
}

char* 
PVM::hostname(int host) 
{ 
    init_vmdata(); 
    validate_hostnum(host); 
    return hostp[host].hi_name; 
}

char* 
PVM::hostname() 
{ 
    return hostname(this_host()); 
}

void 
PVM::init_vmdata()
{
    if(!need_vmdata) return;
    assert(pvm_config(&nhost,&narch,&hostp)>=0, "init_vmdata");
    dtid=pvm_tidtohost(pvm_mytid());
    for(int i=0;i<nhost;i++)
	if(hostp[i].hi_tid==dtid) 
	    { ourhost=i; break; }
    need_vmdata=false;
}

inline void
PVM::validate_hostnum(int n) 
{
    assert(n>=0 && n<num_hosts(),"validate_hostnum -- invalid host number.");
}


void 
PVM::assert(bool b, char* error)
{
    if(!b) die(error);
}

void 
PVM::die(const char* msg)
{
    int buflen=strlen(msg)+30;
    char buf[buflen];
    ostrstream ost(buf,buflen);
    ost<<"PVM("<<exname<<") error  "<<msg;
    pvm_perror(buf);
    exit(-1);	
}

