// pvm.h

#include <pvm3.h>

class PVM 
{
public:
    // start and end
    PVM(char* executable_name);
    ~PVM();
    void kill(char* msg="unknown error");

    // receive messages
    int   wait_next_mesg(int tag=-1);// ret val can be used to get additional info.
    int   unpack_int();
    int*  unpack_intarray(int len);
    void  unpack_str(char* buf);

    // send messages
    void  init_send(int encoding=PvmDataDefault);
    void  pack_int(int n);
    void  pack_str(char* s);
    void  send_to_parent(int tag=0);
    void  send_to_task(int taskid, int tag=0);

    // relationship to other tasks
    int   taskid() { return mytid; }
    bool  noparent() ; 
    int   parent_taskid();

    // create child tasks, running as the same executable.
    void  spawn(int children[], int ntask=1, char* argv[]=(char**)NULL);
    void  spawn_onhost(int children[], char* hname, int ntask=1, char* argv[]=(char**)NULL);

    // info about the virtual machine
    int   num_hosts() { init_vmdata(); return nhost; }
    char* hostname(int hostnum);

    // info about this host.
    int   this_host() { init_vmdata(); return ourhost; }
    char* hostname();

private:
    int   mytid;		// this processes pvm task id.
    char* exname;		// the name of this executable

    // virtual machine data 
    void  init_vmdata();
    void  validate_hostnum(int n);
    bool  need_vmdata;		// true <==> still need to initialize vmdata. 
    int   nhost, narch, dtid, ourhost;
    struct pvmhostinfo* hostp;

    void assert(bool b, char* error); // quits with error b false
    void die(const char* msg);
};

