// pvmtables.cc 

#include "pvm.h"
#include <iostream.h>
#include <stdlib.h>

#include "h1nchi.h"
#include "display.h"
#include "charpoly.h"
#include "ZZX.h"
#include "ZZXFactoring.h"


const TAG=1;
const bool SAVE=true;

const int TASK_CHARPOLY=1;
const int TASK_EIGENFORMS=2;

void master_charpoly(PVM&, int, int);
void master_eigenforms(PVM&, int, int);

void help() 
{
    cout << "PVM Table functions:"<<endl
	 << " e:       compute dual basis of eigenforms for database." <<endl
	 << " cp:      compute characteristic polynomials."<<endl
	 << endl << endl ;
}

void master(PVM& pvm)
{
    cout << endl
	 << "************************************************************" << endl
	 << "   Create Tables using Parallel Virtual Machine and Hecke" << endl
	 << "************************************************************" << endl << endl ;
    
    while(1) {
	char buf[256];
	get_str(buf, 256, "? ");
	if (strcmp(buf,"x")==0 || strcmp(buf,"q")==0) break;
	if (strcmp(buf,"?")==0) { 
	    help();
	} else if (strcmp(buf,"e")==0) {
	    int Nstart=get_int("N start = ");
	    int Nstop=get_int("N stop = ");
	    master_eigenforms(pvm, Nstart, Nstop);
	} else if (strcmp(buf,"cp")==0) {
	    master_charpoly(pvm, get_int("N start = "), get_int("N stop = "));
	}
    }
}

void spawn_charpoly(PVM& pvm, int N, char* hname=0)
{
    cerr << "Spawning child for N="<<N<<endl;
    int child;
    if(hname)
	pvm.spawn_onhost(&child,hname);
    else
	pvm.spawn(&child);
    pvm.init_send();
    pvm.pack_int(TASK_CHARPOLY);
    pvm.pack_int(N);
    pvm.send_to_task(child,TAG);
}

void master_charpoly(PVM& pvm, int Nstart, int Nstop)
{
    int nh=pvm.num_hosts();
    int N=Nstart;
    for(;N<Nstart+nh;N++)
	spawn_charpoly(pvm, N);

    char fname[256];
    ostrstream os(fname,256);
    os<<"charpoly_s2_"<<Nstart<<"-"<<Nstop<<".gp"<<'\0';
    ofstream out(userdata_write(fname));
    if(!out) die("master_charpoly -- failed to open output file.");
    out<<"\\\\ "<<fname<<endl;
    for(;N<=Nstop+nh;N++) {
	pvm.wait_next_mesg(TAG);

	char hname[256];
	pvm.unpack_str(hname);
	if(N<=Nstop) spawn_charpoly(pvm,N,hname);

	int buflen=pvm.unpack_int();
	if(buflen<0) die("master_charpoly -- error in child!");
	char* buf=new char[buflen];
	pvm.unpack_str(buf);
	out<<buf;
	delete buf;
    }
}

void slave_charpoly(PVM& pvm)
{
    int N=pvm.unpack_int();

/*
    char fname[256];
    ostrstream os(fname,256);
    os<<"dbg"<<N<<'\0';
    ofstream dbg(userdata_write(fname));
    dbg<<"Hi I'm slave on " << pvm.hostname() << ", doing N="<<N<<endl;
*/

    Character chi (N,0,SAVE) ;
    H1NChi<bigrational>* H=new H1NChi<bigrational>(chi,2,SAVE);
    if(H==NULL) die("Error in pvmtable allocating H1NCHi object");
    char* buf=NULL;
    if(H->dim_Sk()>0) {
	int buflen=1024;
	buf=new char[buflen];
	ostrstream out(buf,buflen);
	for (int p=2; p<=11; p=next_prime(p)) 
	    if (is_prime(p)) {
		out<<"T["<<N<<","<<p<<"]=";
		Matrix<bigrational> T;
		H->Tn_Sk(p,T);
		ZZX f;
		CharPoly(T,f);
		Display(f,1,1,out);
		out<<";"<<endl;
	    }
	out<<endl<<'\0';
    }
    else {
	buf=new char[1];
	buf[0]='\0';
    }
    delete H;
    pvm.init_send();
    pvm.pack_str(pvm.hostname());
    pvm.pack_int(strlen(buf)+1);
    pvm.pack_str(buf);
    pvm.send_to_parent(TAG);
    delete buf;
}


////////////////////////////////////////////////////////////////
// compute dual basis of eigenforms, but just for the database.

void spawn_eigenforms(PVM& pvm, int N, char* hname=0)
{
    cerr << "spawn_eigenforms";
    if(hname) cerr << "\t\t"<<hname;
    cerr << "\t\tN = "<<N<<endl;
    int child;
    if(hname)
	pvm.spawn_onhost(&child,hname);
    else
	pvm.spawn(&child);
    pvm.init_send();
    pvm.pack_int(TASK_EIGENFORMS);
    pvm.pack_int(N);
    pvm.send_to_task(child,TAG);
}

void master_eigenforms(PVM& pvm, int Nstart, int Nstop)
{
    int nh=pvm.num_hosts();
    int N=Nstart;
    for(;N<Nstart+nh;N++)
	spawn_eigenforms(pvm, N);

    for(;N<=Nstop+nh;N++) {
	pvm.wait_next_mesg(TAG);

	char hname[256];
	pvm.unpack_str(hname);
	if(N<=Nstop) spawn_eigenforms(pvm,N,hname);
    }
}

void slave_eigenforms(PVM& pvm)
{
    int N=pvm.unpack_int();
    Character chi (N,0,SAVE) ;
    H1NChi<bigrational>* H=new H1NChi<bigrational>(chi,2,SAVE);
    if(H==NULL) die("Error in pvmtable allocating H1NCHi object");
    Matrix<bigrational> V; H->get_Sknew(V);
    if(V.ncols()>0) {
	MatVecList<bigrational> E;
	H->DualBasisOfEigenforms(E,V);
    }
    delete H;
    pvm.init_send();
    pvm.pack_str(pvm.hostname());
    pvm.send_to_parent(TAG);
}
////////////////////////////////////////////////////////////











void slave(PVM& pvm) 
{
    pvm.wait_next_mesg(TAG);
    int task = pvm.unpack_int();

    switch(task) {
    case TASK_CHARPOLY:
	slave_charpoly(pvm);
	return;
    case TASK_EIGENFORMS:
	slave_eigenforms(pvm);
	return;
    };

    pvm.init_send();
    pvm.pack_int(-1);
    pvm.send_to_parent(TAG);
}

int 
main(int argc, char* argv[])
{
    PVM pvm(argv[0]);

    if (pvm.noparent()) {
        master(pvm);
    } else 
	slave(pvm);
    return 0;
}

