/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu

    This source file is based on code of John Cremona and Luiz. 
------------------------------------------------------------------------- */

#include "charmod.h"

int charmod::pr =3;
int charmod::halfpr =1;
int charmod::prim_root=2 ;

void 
charmod::set_pr(int p)
{
    if( p > 251 ) { cout << "prime is too big!"; exit(-1); }
    if( p < 3 ) { cout << "prime is too small"; exit(-1); }
    pr = p;
    halfpr = pr/2; 

    // compute a primitive root mod p.
    // dumb algorithm, but o.k. since p <= 251.
    for (prim_root = 2 ; prim_root < p ; prim_root++) {
      int i ;
      for (i = 2 ; i <= (p-1)/2 ; i++)
	if ( ((p-1) % i == 0) && powmod (prim_root, i, p) == 1 ) 
	  i = p ;
      if (i < p) 
	break ;
    }
    if (p!= 2 && (prim_root == p || powmod (prim_root, p-1, p) != 1 )) {
      cerr << "Error in charmod::set_pr -- no primitive root." << endl ;
      exit(-1) ;
    }
}

void 
charmod::error(const char* msg) const
{
    cerr << "Error in class charmod:\n" << msg << endl;
}

charmod inv(const charmod& n) 
{
    int x,y;
    int g = Bezout( n.val, charmod::pr, x, y );
    if( g == 1 ) return x;
    else n.error("inv called for non-invertible number");
}

ostream& operator<< (ostream& s, const charmod& n)
{
    s << int(n.val);
    return s;
}

istream& operator>> (istream& s, charmod& n)
{
    int i;
    s >> i;
    i %= charmod::pr;
    if( i < 0 ) i += charmod::pr;
    n.val = i;
    return s;
}


