/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu
------------------------------------------------------------------------- */

/* 
   FILE: charpoly.h 
   DESC: routines for computing and displaying characteristic polynomials.

   NOTES:
   1. For characteristic polynomial computation I use Schoop's NTL because
      it's the fastest thing I've found so far.  It's much faster than
      LiDIA or PARI for computing char polys. 

   2. Though Luiz's Matrix template class has a build in function for
      computing characteristic polynomials, when templated with the 
      Rational class it is very slow. 
      
   3. In computing the characteristic polynomial over Z the program converts 
      a Matrix<Rational> to a matrix(ZZ) (after multiplying through by the
      least common multiple.)  The entries of the Rational matrix are
      "carried" to the matrix(ZZ) as longs so if the entries of the 
      Rational matrix are too large, an error message will be printed
      and the program will be terminated. 
   --------------------------------------------------------------------- */

#ifndef CHARPOLY_H
#define CHARPOLY_H

#include "matrix.h"
#include "NTL/mat_ZZ.h"
#include <LiDIA/bigrational.h>
class charmod ;
class intmod ;
class ZZ_pX;
class ZZX ;

void Matrix_ZZify(mat_ZZ& A, bigint& l, const Matrix<bigrational>& M);
void CharPoly (Vector<bigrational>& f, const Matrix<bigrational>& M) ;
bigrational Determinant (const Matrix<bigrational>& M);

// STUPID: I made a mistake with the order of the return parameter when I
//         write this code. 
void CharPoly (const Matrix<bigrational>& M, ZZX& pol) ;

// in order to make a ZZ_pX it is necessary to 
// first set the ZZ_p modulus like so:
//  ZZ_pInit (charmod::get_pr()) ;    // or intmod
void CharPoly (const Matrix<charmod>& M, ZZ_pX& pol) ;
void CharPoly (const Matrix<intmod>& M, ZZ_pX& pol) ;

// returns a nonzero value iff the roots of pol are distinct,
// i.e., iff gcd(pol, pol')  is constant. 
bool DiscIsZero (const ZZ_pX& pol) ;
bool DiscIsZero (const ZZX& pol) ;

#endif // CHARPOLY_H

