/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu
------------------------------------------------------------------------- */

//   cusps.h

#include <LiDIA/bigint.h>
#include "array.h"
#include "character.h" 

// This class represents the space of cusp classes H_0^{+}(G,Chi).
//
// [alpha] = [-alpha]
// [gamma_u(alpha)] = chi(u)[alpha]   
// [alpha] = [gamma.alpha] for all gamma in Gamma_1(N). 
//

class Cusps 
{
    Array<bigint> g1_p, g1_q, g1_u ;
    Array<bigint> _list_p, _list_q, _list_u ;
    bool _list_add (bigint p, bigint q, bigint u) ;
    bool is_cusp_killed (bigint p, bigint q) ;
    rootof1 _chi_u (bigint p, bigint q) ;
    
    
    struct matrix {
	bigint a, b, c, d;
    } ;
    
    matrix* gam ;
    void _init_gamma() ;
    int phiN ;
    
    Character chi ;
    int N ;
    int k ;
    bool plus;
    
    Array<bigint> g0_p, g0_q;
    Array<int> g0_is_zero ;
    int num_cusps ;
    bool equiv_signed (bigint p1, bigint q1, bigint p2, bigint q2) ;
    bool equiv (bigint p1, bigint q1, bigint p2, bigint q2) ;
    
public:
    
    Cusps (const Character& chi, int k=2, bool plus=true) ;
    ~Cusps();
    // [p/q] = coeff*(idx), (idx) = idx-th cusp. 
    void free (bigint p, bigint q, int& idx, rootof1& coeff) ;
    // returns the number of distinct cusps we know about so far.
    int number_of_cusps () { return num_cusps ; }
} ;







