/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu
------------------------------------------------------------------------- */

// display.cc 
#include "display.h"
#include "charpoly.h"
#include "charmod.h"
#include "NTL/ZZXFactoring.h"
#include "NTL/ZZ_pXFactoring.h"
#include "NTL/ZZX.h"
#include "NTL/ZZ_p.h"
#include "NTL/ZZ_pX.h"
#include "charmod.h"
#include "intmod.h"

void DisplayCharPoly (const Matrix<algnum>& M) 
{
  Vector<algnum> pol;
  M.charpoly(pol);

//  pol.polprint(); cout<<endl;

  VecFac<algnum> fac;
  pol.polfactor(fac);
  cout<<endl;
  for(int i=1;i<=fac.numfactors();i++) {
      cout << "(" ; fac.base(i).polprint();
      cout << ")^" << fac.exp(i) ;
      if(i<fac.numfactors()) cout<<" * ";
  }
  
  /*
  int d = pol.size()-1 ;
  if (d < 0) {
    cout << "0";
    return ;
  }
  for(int i=d; i>=0; i--) {
    if(pol[i+1] != algnum(0)) {
      if (pol[i+1] == algnum(-1) && i>0) 
	cout << "-" ;
      if ((pol[i+1] != algnum(1) && pol[i+1]!=algnum(-1)) || i==0)  {
	cout << "(" << pol[i+1] << ")" ;
	if (i>0) cout << "*";
      }
      if (i>1)
	cout << "T^" << i << " ";
      if (i==1)
	cout << "T " ;
    }
    if(i>0)
      cout << "+ ";
  }
  */
}

void DisplayCharPoly (const Matrix<ffield_element>& M) 
{
  cout << "DisplayCharPoly(const Matrix<ffield_element>) not implimented."
       <<endl;
  // The following won't work because it requires dividing, possibly
  // by the characteristic. 
  /*
  Vector<ffield_element> poly;
  M.charpoly(poly);
  cout << poly ;
  */
}

void DisplayCharPoly (const Matrix<bigrational>& M)
{
  ZZX f ;
  CharPoly (M, f);
  Display (f,1,1) ;
}

void DisplayCharPoly (const Matrix<charmod>& M)
{    
  ZZ_p::init(ZZ(INIT_VAL,charmod::get_pr()));

  ZZ_pX f;
  CharPoly(M, f);
  Display(f,1,1) ;
}

void DisplayCharPoly (const Matrix<intmod>& M)
{    
  ZZ_p::init (ZZ(INIT_VAL,intmod::get_pr())) ;
  ZZ_pX f ;
  CharPoly (M, f);
  Display (f,1,1) ;
}


void _Display (const ZZX& pol, int mode=0, ostream& os=cout) 
{
  int d = deg(pol) ;
  if (d == -1) {
    os << "0";
    return ;
  }
  for(int i=d; i>=0; i--) {
    if(coeff(pol,i) != 0) {
      if (coeff(pol,i) == -1 && i>0) 
	os << "-" ;
      if (abs(coeff(pol,i)) != 1 || i==0)  {
	os << coeff(pol,i) ;
	if (i>0 && mode) os << "*";
      }
      if (i>1) {
	os << "x^"; 
	if(mode || i<10) os<<i;
	else os<<"{"<<i<<"}";
	os << " ";
      }
      if (i==1)
	os << "x " ;
    }
    if(i>0 && coeff(pol,i-1) > 0)
      os << "+ ";
  }
}

void _Display (const ZZ_pX& pol, int mode=0, ostream& os=cout) 
{
  int d = deg(pol) ;
  if (d == -1) {
    os << "0";
    return ;
  }
  for(int i=d; i>=0; i--) {
    if(coeff(pol,i) != 0) {
      if ((coeff(pol,i) != 1 && coeff(pol,i)!=-1) || i==0)  {
	os << coeff(pol,i) ;
	if (i>0 && mode) os << "*";
      }
      if (i>1) {
	os << "x^"; 
	if(mode || i<10) os<<i;
	else os<<"{"<<i<<"}";
	os << " ";
      }
      if (i==1)
	os << "x " ;
    }
    if(i>0 && coeff(pol,i-1) != 0)
      os << "+ ";
  }
}

void DisplayPol (const Vector<bigrational>& pol, int mode, int do_factor, ostream& os)
{
    ZZX g;
    bigint denom;
    to_ZZX (g, denom, pol);
    if(denom!=1)
	os << "(";
    Display (g, mode, do_factor, os);
    if(denom!=1) os << ")/" << denom;

}

void Display (const ZZX& pol, int mode, int do_factor, ostream& os) 
{
  if (do_factor) {
    vec_pair_ZZX_long factors ;
    ZZ c ;
    factor (c, factors, pol) ;

    if (c != 1) {
      os << c ;
      if (mode) 
	os << "*";
    }

    for (int i = 1 ; i <= factors.length() ; i++) {
      if (factors.length() > 1 || factors(i).b>1)
	os << "(" ;
      _Display (factors(i).a, mode, os) ;
      if (factors.length() > 1 || factors(i).b>1)
	os << ")" ;
      if (factors(i).b > 1) {
	os << "^";
	if (mode || i<10) os << factors(i).b ;
	else os << "{" << factors(i).b << "}";
      }
      if (mode && i<factors.length())
	os << "*" ;
    } 
  } else {
    _Display (pol, mode, os) ;
  }
}

void Display (const ZZ_pX& pol, int mode, int factor, ostream& os) 
{
  if (coeff(pol, deg(pol)) != 1 && factor) {
    cerr << "factored display (ZZ_pX pol) only implemented for pol monic." << endl;
    exit(0) ;
  }
    
  if (factor) {
    vec_pair_ZZ_pX_long factors ;
    berlekamp (factors, pol) ;
    for (int i = 1 ; i <= factors.length()  ; i++) {
      //      if (factors.length() > 1 || factors(i).b>1)
      os << "(" ;
      _Display (factors(i).a, mode, os) ;
      //      if (factors.length() > 1 || factors(i).b>1)
      os << ")" ;
      if (factors(i).b>1) {
	os << "^";
	if (mode || i<10) os << factors(i).b ;
	else os << "{" << factors(i).b << "}";
      }
      if (mode && i<factors.length())
	os << "*" ;
    }
  } else {
    if (mode == 1)
      os << "(" ;
    _Display(pol, mode, os) ;
    if (mode == 1)
      os << ")" ;
  }

  if (mode == 1)
    os << "*Mod(1," << ZZ_p::modulus() << ")" ;

}



void Display (const Matrix<algnum>& M, int mode=1, ostream& os) 
{
  os << "[";
  for (int r = 1 ; r <= M.nrows() ; r++)
    for (int c = 1 ; c <= M.ncols() ; c++) {
      os << M(r,c) ;
      if (c < M.ncols()) 
	os << "," ;
      else if (r < M.nrows())
	os << ";" << endl << " ";
      else
	os << "]" << endl ;
    }
}

void Display (const Matrix<bigcomplex>& M, int mode=1, ostream& os) 
{
  os << "[";
  for (int r = 1 ; r <= M.nrows() ; r++)
    for (int c = 1 ; c <= M.ncols() ; c++) {
      Display(M(r,c)) ;
      if (c < M.ncols()) 
	os << "," ;
      else if (r < M.nrows())
	os << ";" << endl << " ";
      else
	os << "]" << endl ;
    }
}


void Display (const Matrix<ffield_element>& M, int mode=1, ostream& os) 
{
  os << "[";
  for (int r = 1 ; r <= M.nrows() ; r++)
    for (int c = 1 ; c <= M.ncols() ; c++) {
      os << M(r,c) ;
      if (c < M.ncols()) 
	os << "," ;
      else if (r < M.nrows())
	os << ";" << endl << " ";
      else
	os << "]" << endl ;
    }
}


void Display (const Matrix<bigrational>& M, int mode=1, ostream& os) 
{
  os << "[";
  for (int r = 1 ; r <= M.nrows() ; r++)
    for (int c = 1 ; c <= M.ncols() ; c++) {
      os << M(r,c) ;
      if (c < M.ncols()) 
	os << "," ;
      else if (r < M.nrows()) {
	os << ";";
	if(mode==1)
	  os << endl << " ";
      }
      else {
	os << "]" ;
	if(mode==1) os<<endl;
      }
    }
}

void Display (const Matrix<charmod>& M, int mode=1, ostream& os) 
{
  os << "[";
  for (int r = 1 ; r <= M.nrows() ; r++)
    for (int c = 1 ; c <= M.ncols() ; c++) {
      os << M(r,c) ;
      if (c < M.ncols()) 
	os << "," ;
      else if (r < M.nrows())
	os << ";" << endl << " ";
      else
	os << "]" << endl ;
    }
}

void Display (const Matrix<intmod>& M, int mode=1, ostream& os) 
{
  os << "[";
  for (int r = 1 ; r <= M.nrows() ; r++)
    for (int c = 1 ; c <= M.ncols() ; c++) {
      os << M(r,c) ;
      if (c < M.ncols()) 
	os << "," ;
      else if (r < M.nrows())
	os << ";" << endl << " ";
      else
	os << "]" << endl ;
    }
}

void _DisplayFactored (const bigint& N, ostream& os) 
{
  if(N==0 || N==1) { os << N; return; }
  if(N<0) os<<"-";
  rational_factorization fac(N);
  fac.factor();
  for(int i=0; i<fac.no_of_comp(); i++) {
    if (i>0) os << "*";
    os << fac.base(i) ; 
    if (fac.exponent(i)>1) 
      os << "^" << fac.exponent(i) ;
  }
}

void DisplayFactored (const bigrational& x, ostream& os=cout) 
{
    _DisplayFactored(numerator(x),os);
    if(denominator(x)!=1) {
	os<<"/";
	_DisplayFactored(denominator(x),os);
    }
}


int Save (const Matrix<charmod>& M, const char* fname) 
{
  ofstream of(fname) ;
  if (!of) return 0 ;
  of << M.nrows () << ' ' << M.ncols () << endl ;

  for (int r = 1 ; r <= M.nrows () ; r++ )
    for (int c = 1 ; c <= M.ncols () ; c++) 
      of.put (ulift(M(r,c))) ;
}


int Load (Matrix<charmod>& M, const char* fname) 
{
  ifstream in (fname) ;
  if (!in) return 0 ;
  int rows, cols ;
  in >> rows >> cols ;
  char u = 0 ;
  while (u != '\n') 
    in.get (u) ;
  M.init (rows, cols) ;
  for (int r = 1 ; r <= rows ; r++ ) {
    for (int c = 1 ; c <= cols ; c++) {
      char u ; 
      in.get (u) ;
      M (r,c) = u ;
    }
  }
  return 1 ; 
}

void TexDisplay(const bigint& N,
		ostream& os)
{
  if(N==0 || N==1) { os << N; return; }
  if(N<0) os<<"-";
  rational_factorization fac(N);
  fac.factor();
  for(int i=0; i<fac.no_of_comp(); i++) {
    if (i>0) os << "\\cdot";
    os << fac.base(i) ; 
    if (fac.exponent(i)>=10) 
	os << "^{" << fac.exponent(i)<< "}" ;
    else if(fac.exponent(i)>1)
	os << "^" << fac.exponent(i) ;
  }
}

void texprintbigrational(ostream& out,
			 const bigrational& x)
{
  if(denominator(x)==1) { TexDisplay(numerator(x),out); return; }
  if(x<0)out<<"-";
  out<<"\\frac{";TexDisplay(numerator(abs(x)),out);
  out<<"}{"; TexDisplay(denominator(abs(x)),out);
  out<<"}";
}

void texprintalgnum (ostream& out,
		     Vector<bigrational>& coeff, 
		     const char* var) 
{
  bool allzero=true;
  for(int i=1; i<=coeff.size(); i++)
    if (coeff[i]!=0) {
      if(!allzero && coeff[i]>0)
	out<<"+";
      allzero=false;
      if(coeff[i]==-1 && i>1) 
	out<<"-";
      else
	if(i==1||i>1&&coeff[i]!=1)
	  texprintbigrational(out,coeff[i]);
      if(i>1)
	out<<var;
      if(i>2)
	out<<"^{"<<i-1<<"}";
    }
  if (allzero) out<<"0";
}

void texprintpoly   (ostream& out,
		     Vector<bigrational>& coeff,
		     const char* var) 
{
  out<<var<<"^{"<<coeff.size()<<"}";
  for(int i=coeff.size(); i>=1; i--)
    if (coeff[i]!=0) {
      if(coeff[i]>0)
	out<<"+";
      if(coeff[i]==-1 && i>1) 
	out<<"-";
      else
	if(i==1||i>1&&coeff[i]!=1)
	  texprintbigrational(out,coeff[i]);
      if(i>1)
	out<<var;
      if(i>2)
	out<<"^{"<<i-1<<"}";
    }
  out<<"=0";
}

void nonmonic_pari_printpoly(Vector<bigrational>& coeff,
			     ostream& out,
			     const char* var) 
{
  bool first=true;
  for(int i=coeff.size(); i>=1; i--)
    if (coeff[i]!=0) {
      if(coeff[i]>0 && !first)
	out<<"+";
      if(coeff[i]==-1 && i>1) 
	out<<"-";
      else
	if(i==1||i>1&&coeff[i]!=1) {
	  out<<coeff[i];
	  if (i>1) out<<"*";
	}
      if(i>1) 
	out<<var;
      if(i>2)
	out<<"^"<<i-1<<"";
      first=false;
    }
  if(first) out<<"0";
}

void Display (const int& x, ostream& os=cout)
{
    os << x;
}

void Display (const bigrational& x, ostream& os)
{
    DisplayFactored(x,os);
}

void Display (const bigint& x, ostream& os)
{
    _DisplayFactored(x,os);
}
                                         

void TexDisplay (const bigrational& x, ostream& os)
{
    texprintbigrational(os, x);
}

void TexDisplay (const bigcomplex& x, ostream& os)
{
    if(real(x) != 0)
	os << real(x);
    if(real(x) != 0 && imag(x) != 0)
	os << " + ";
    if(imag(x) != 0)
	os << imag(x) << "i";
}


void Display (charmod& x, ostream& os) { os<<x;}
void Display (intmod& x, ostream& os)  { os<<x;}
void Display (algnum& x, ostream& os) { os<<x;}
void Display (ffield_element& x, ostream& os)  { os<<x;}
void Display (const bigcomplex& x, ostream& os)
{ 
    if(real(x)==0 && imag(x)==0) { os << "0"; return; }
    if(real(x) != 0)
	os << real(x);
    if(real(x) != 0 && imag(x) != 0)
	os << " + ";
    if(imag(x) != 0)
	os << imag(x) << "*I";
}

void TexDisplay (charmod& x, ostream& os=cout) 
{
    mesgln("TexDisplay(charmod): not yet written.");
}

void TexDisplay (intmod& x, ostream& os=cout)
{
    mesgln("TexDisplay(intmod): not yet written.");
}

void TexDisplay (algnum& x, ostream& os=cout)
{
    mesgln("TexDisplay(algnum): not yet written.");
}

void TexDisplay (ffield_element& x, ostream& os=cout)
{
    mesgln("TexDisplay(ffield_element): not yet written.");
}


void DisplayAbGp (const Vector<bigint>& v, ostream& os=cout)
{
    if(v.size()==0) {
	os<<"{0}";
	return;
    }
    for(int i=1; i<=v.size(); i++) {
	os<<"Z/"; Display(v[i],os);
	if(i<v.size()) os<<" + ";
    }
}

void DisplayRndVec (const Vector<bigrational>& v, ostream& os=cout)
{
    os<<"(";
    for(int i=1; i<=v.size(); i++) {
	Display(v[i],os);
	if(i<v.size()) os<<",";
    }
    os<<")";
}

void DisplayRndVec (const Vector<bigint>& v, ostream& os=cout)
{
    os<<"(";
    for(int i=1; i<=v.size(); i++) {
	Display(v[i],os);
	if(i<v.size()) os<<",";
    }
    os<<")";
}


void TexDisplayAbGp (const Vector<bigint>& v, ostream& os=cout)
{
    for(int i=1; i<=v.size(); i++) {
	os<<"\\Z/"; TexDisplay(v[i],os);
	if(i<v.size()) os<<"+";
    }
}

void TexDisplayRndVec (const Vector<bigrational>& v, ostream& os=cout)
{
    os<<"(";
    for(int i=1; i<=v.size(); i++) {
	TexDisplay(v[i],os);
	if(i<v.size()) os<<",";
    }
    os<<")";
}

void TexDisplayRndVec (const Vector<bigint>& v, ostream& os=cout)
{
    os<<"(";
    for(int i=1; i<=v.size(); i++) {
	TexDisplay(v[i],os);
	if(i<v.size()) os<<",";
    }
    os<<")";
}

void DisplayQExpansion(const Matrix<bigrational>& Q, const Vector<bigrational>& f,
		       int prec, const char* var="a", bool tex=false,
		       ostream& out=cout)
{
    int m=min(prec,Q.ncols());
    for(int n=1; n<=m; n++) {
	const Vector<bigrational>& a=Q.col(n);
	if(!a.iszero()) {
	    if(!(a.numnonzero()==1 && a[1]==1)) {
		if(a.numnonzero()>1) out << "(";
		a.polprint(out,var,false,tex);
		if(a.numnonzero()>1) out << ")";
		if(!tex) out<<"*";
	    }
	    out << "q"; 
	    if(n>1) {
		out << "^";
		if(tex && n>=10) out << "{";
		out << n;
		if(tex && n>=10) out << "}";
	    }
	    out << " + ";
	}
    }
    if(tex) out << "\\cdots"; else out << "O(q^"<<m+1<<")";
}

void DisplayQExpansion(const Matrix<bigrational>& Q, 
		       int prec, const char* var="q", bool tex=false,
		       ostream& out=cout)
{
    Vector<bigrational> f(2); f[1]=0; f[2]=1;
    for(int n=1; n<=min(prec,Q.nrows()); n++) {
	out<<"f" << n << " = ";
	DisplayQExpansion(Matrix<bigrational>(Q.row(n)),
			  f,prec,var,tex,out);
	out<<endl;
    }

}



