/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu
------------------------------------------------------------------------- */

/* 
 FILE: display.h 
 DESC: Display various objects arising in my computations using
       either TeX or pari readable format. 
*/

#ifndef DISPLAY_H
#define DISPLAY_H

#include "fields.h"
#include <iostream.h>
#include "matrix.h"
#include <LiDIA/bigrational.h>
class charmod;
class intmod;
class ZZX;
class ZZ_pX;


// display pol in a pretty (pari-readable) format. 
// if factor != 0 then the poly is displayed in factored form.
// mode = 1 is pari readable.
// mode = 0 is TeX readable.
void Display (const ZZX& pol, int mode=1, int factor=0, ostream& os=cout) ;
void Display (const ZZ_pX& pol, int mode=1, int factor=0, ostream& os=cout) ;

void DisplayPol (const Vector<bigrational>& pol, int mode=1,
	      int factor=0, ostream& os=cout) ;


// Display matrices
// mode = 1 = pari        * this is the only one which is currently implemented for all types
// mode = 2 = pari, one line
void Display (const Matrix<bigrational>& M, int mode=1, ostream& os = cout) ;
void Display (const Matrix<charmod>& M, int mode=1, ostream& os = cout) ;
void Display (const Matrix<intmod>& M, int mode=1, ostream& os = cout) ;
void Display (const Matrix<algnum>& M, int mode=1, ostream& os = cout) ;
void Display (const Matrix<ffield_element>& M, int mode=1, ostream& os = cout) ;
void Display (const Matrix<bigcomplex>& M, int mode=1, ostream& os = cout) ;

// Display char polys
void DisplayCharPoly (const Matrix<bigrational>& M);
void DisplayCharPoly (const Matrix<charmod>& M);
void DisplayCharPoly (const Matrix<intmod>& M);
void DisplayCharPoly (const Matrix<algnum>& M);
void DisplayCharPoly (const Matrix<ffield_element>& M);

// Display numbers
void Display (const int& x, ostream& os=cout); 
void Display (const bigint& x, ostream& os=cout);
void Display (const bigrational& x, ostream& os=cout);
void Display (charmod& x, ostream& os=cout);
void Display (intmod& x, ostream& os=cout);
void Display (algnum& x, ostream& os=cout);
void Display (ffield_element& x, ostream& os=cout);
void Display (const bigcomplex& x, ostream& os=cout);
void DisplayAbGp (const Vector<bigint>& v, ostream& os=cout);
void DisplayRndVec (const Vector<bigrational>& v, ostream& os=cout);
void DisplayRndVec (const Vector<bigint>& v, ostream& os=cout);

void TexDisplay (const bigint& N, ostream& os=cout);
void TexDisplay (const bigrational& x, ostream& os=cout);
void TexDisplay (charmod& x, ostream& os=cout);
void TexDisplay (intmod& x, ostream& os=cout);
void TexDisplay (algnum& x, ostream& os=cout);
void TexDisplay (ffield_element& x, ostream& os=cout);
void TexDisplay (const bigcomplex& x, ostream& os=cout);
void TexDisplayAbGp (const Vector<bigint>& v, ostream& os=cout);
void TexDisplayRndVec (const Vector<bigrational>& v, ostream& os=cout);
void TexDisplayRndVec (const Vector<bigint>& v, ostream& os=cout);

				// *efficiently* store charmod matrices.
int Save (const Matrix<charmod>& M, const char* fname) ;
int Load (Matrix<charmod>& M, const char* fname) ;

void _DisplayFactored (const bigint& N, ostream& os=cout) ;
void DisplayFactored (const bigrational& x, ostream& os=cout) ;

void texprintalgnum (ostream& out,
		     Vector<bigrational>& coeff, 
		     const char* var) ;
void texprintpoly   (ostream& out,
		     Vector<bigrational>& coeff,
		     const char* var) ;
void texprintbigrational(ostream& out,
			 const bigrational& x);

// if v.size()==3 and var=="x", 
//       sends "v[3]*x^2 + v[2]*x + v[1]" to out.
void nonmonic_pari_printpoly  (Vector<bigrational>& v,
			       ostream& out=cout,
			       const char* var="x") ;

void DisplayQExpansion(const Matrix<bigrational>& Q, const Vector<bigrational>& f,
		       int prec, const char* var="a", bool tex=false,
		       ostream& out=cout);

void DisplayQExpansion(const Matrix<bigrational>& Q, 
		       int prec, const char* var="q", bool tex=false,
		       ostream& out=cout);

#endif // DISPLAY_H




