/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu
------------------------------------------------------------------------- */

/* 
 FILE: example.cc
 DESC: Example of how to use the HECKE C++ libary.
*/

#include <stdlib.h>

#define SAVE     true

#include "h1nchi.h"
#include "system.h"
#include "modformulas.h"
#include "display.h"
#include "methodofgraphs.h"

void example1 () 
{
    cout << "Example."<<endl;
    while(true) {
	cout << endl << endl << "p = ";
	int p; 
	cin >> p;
	intmod::set_pr(p);

	H1NChi<intmod> H(Character(1, -1, true), p+1, true, true);
	cout << "Created S_{" << p+1 << "}(SL_2(Z)) over F_"<<p<<endl;
        cout << "dim = " << H.dim_Sk()<< endl;
        if(H.dim_Sk()==0)
           cout << "dim = 0" << endl;
        else {
	   Matrix<intmod> T;
           for(int ell=2;ell<=7; ell=next_prime(ell)) {
              cout << "T_" << ell <<" = " << T << endl;
	      H.Tn_Sk (ell, T);
              cout << endl;
	      DisplayCharPoly(T);
           }
        }
    }
}

void example2()
{
    cout << "Example 2" << endl;
    int N=get_int("N = ");
    MethodOfGraphs m(N,true);
    Matrix<bigrational> T;
    while(1) {    
	cout << "Compute charpoly Tp modulo ell"<<endl;    
	int p = get_int("p = ",2,7);
	int ell = get_int("ell = ",3,251);
	if(!is_prime(p) || !is_prime(ell)) continue;
	m.Tp(T,p);
	charmod::set_pr(ell);
	Matrix<charmod> Tl;
	to_charmod(Tl, T);
	cout << "T_" << p << " modulo " << ell << endl;
	DisplayCharPoly(Tl); 
    }                  
}

void example3()
{
    cout << "Example 3: character group modular degrees "
	 << "of factors of J_0(p)" << endl;

    int p = get_int("p = ");
    cout << "Enter a prime ell <= 7, such that the factor you"<<endl
	 << "wish to analyze occurs with multiplicity one in the"<<endl
	 << "characteristic polynomial of T_ell."<<endl;
    int ell;
    do {
	ell = get_int("ell = ",2,7);
    } while (!is_prime(ell));
    
    // compute and factor charpoly T_ell:
    MethodOfGraphs m(p,true);
    Matrix<bigrational> T;
    m.Tp(T,ell);
    Vector<bigrational> f;
    T.charpoly(f);
    VecFac<bigrational> fac;
    f.polfactor(fac);
    while(1) {
	cout<<endl;
	for(int i=1;i<=fac.numfactors();i++) {
	    cout << "\t" << i << ": " ; 
	    if(fac.exp(i)>1) cout << "(" ;
	    fac.base(i).polprint();
	    if(fac.exp(i)>1) cout << ")^" << fac.exp(i);
	    cout << endl;
	}
	int n=get_int("Select a factor: ",0,fac.numfactors());
	if(n==0) break;

	// compute m_x and Phi_X:
	IntVecList<bigrational> I;
	I.init(1);
	I.Z(0)=ell; I.v(0)=fac.base(n);
	bigint image, mx;
	m.coker_mx(image, mx, I);
	cout << "Phi_X = " << image << ",  mx = " << mx << endl;
    }
}

void example4()
{
    cout << "Example 4" << endl;
    cout << "Compute the number of factors of T_2 modulo p" << endl;
    int pstart=get_int("pstart = ");
    if(!is_prime(pstart)) pstart=next_prime(pstart);
    int pstop=get_int("pstop = ");
    if(!is_prime(pstop)) pstop=next_prime(pstop);
    for(int N=pstart; N<=pstop; N=next_prime(N)) {
	MethodOfGraphs m(N,true);
	Matrix<bigrational> T;
	m.Tp(T,2);
	InitFieldIntmod(N);
	Matrix<intmod> Tl;
	to_intmod(Tl, T);
	Vector<intmod> pol;
	Tl.charpoly(pol);
	VecFac<intmod> fac;
	pol.polfactor(fac);
	cout << "F[" << N << "] = " << fac.numfactors() << ";" << endl;
    }                  
}


main()
{
    example4();
}
