/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu
------------------------------------------------------------------------- */

/* 
   FILE: hecke.cc
   DESC: Start up the calculators.
*/

#include <stdlib.h>

#define VERSION  "0.4"
#define DATE     "July 9, 1999"
#define SAVE     true
#define EMAIL    "was@math.berkeley.edu"

#include "system.h"
#include "modformulas.h"
#include "methodofgraphs.h"
#include "tables.h"
#include "modularcalc.h"
#include "motive.h"

void bug_list();
void monodromy();
void fun_mesg();

main () 
{
    // fun_mesg();
    cout << "HECKE Version " << VERSION << ", Copyright (C) 1999 William A. Stein" << endl
	 << "HECKE comes with ABSOLUTELY NO WARRANTY."<<endl
	 << "This is free software, and you are welcome to redistribute it"<<endl
	 << "under certain conditions; read the included COPYING file for details."<<endl;
    cout << endl << endl
	 << "          HECKE:  Modular Forms Calculator   Version "
	 << VERSION << " (" << DATE << ")"<<endl<<endl
	 << "                        William Stein"<< endl
	 << "Send bug reports and suggestions to " << EMAIL << "." <<endl
	 << "Type ? for help." << endl <<endl;
    
    while (1) {
	char a[1024], b;
	get_str(a,1024,"HECKE>  ");
	b=a[0];
	if (strcmp(a,"?")==0) {
	    cout << ""<<endl
		 <<"  about:     About"<<endl
		 <<"  calc:      Motive calculator"<<endl
		 <<"  exsymbols: Extended modular symbols mode"<<endl
		 <<"  formulas:  Formula calculator"<<endl
		 <<"  graphs:    Monodromy pairing calculator"<<endl
		 <<"  msymbols:  Modular symbols calculator"<<endl
		 <<"  tables:    Table making routines"<<endl
		 <<"  quit:      Quit"<<endl<<endl;

	} else if (b=='a') {
	    bug_list();
	    cout<<"This program was written by William Stein (was@math.berkeley.edu).\n"
		<<"Kevin Buzzard significantly contributed to design and debugging."<<endl;

	} else if (b=='c') {
	    MotiveCalculator calc;
	    calc.Interface();
	    cout << "Leaving motive calculator."<<endl;

	} else if (b=='f') {
	    ModularFormulas f;
	    f.interactive();
	    cout << "Leaving modular formulas calculator."<<endl;

	} else if (b=='g') {
	    monodromy();
	    cout << "Leaving method of graphs calculator."<<endl;

	} else if (b=='m') {
	    StartupModularCalculator(false, SAVE);
	    cout << "Leaving modular symbols calculator."<<endl;

	} else if (b=='e' || b=='M') {
	    StartupModularCalculator(true, SAVE);
	    cout << "Leaving modular symbols calculator."<<endl;

        } else if (b=='t') {
	    Tables t(SAVE);
	    t.interactive();
	    cout << "Leaving tables calculator."<<endl;

	} else if (strcmp(a,"x")==0 || strcmp(a,"q")==0 || b=='x' || b=='q')
	    break ;

	else
	    cout << "Type ? for help." << endl ;
    }
    cout << "Bye" << endl ;
} 


void fun_mesg()
{
    cout << endl << "Hi " << username() << " welcome to HECKE."<<endl;
    if(strcmp(username(),"helena")==0)
	cout << "Oh, it's Helena Verrill!  Have a great day!"<<endl;
}

void bug_list() 
{
    cout << endl << "** Known bugs in Version " << VERSION << " of hecke **" << endl << endl
	 << "Mathematically incorrect results: " << endl
	 << " * (POSSIBLY FIXED) N=1376, chi=[2,1,3], char=5, k=5, T_5 is computed inconsistently." <<endl
	 <<endl<<endl
	 << "Situations which don't work: " << endl
	 << endl
	 << "Deficiencies to watch out for: " << endl
	 << " * Hecke hasn't been programmed to correctly factor polynomials over"<<endl
	 << "   certain fields.  It just assumes they are irreducible."<<endl
	 << "   This could lead to an error when computing basis of eigenforms."<<endl
	 << endl;
}


void monodromy()
{
    cout << endl << "The Monodromy Pairing via" <<endl
	 << "The Mestre-Oesterle Method of Graphs"<<endl
	 << "Enter the level."<<endl;
    int N;
    while(1) {
	N=get_int("N = ",0);
	if (N==0) return ;
	if (is_prime(N)) break;
	cout<<"N must be prime."<<endl;
    }
    MethodOfGraphs m(N,SAVE);
    m.interface();
}
