/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu
------------------------------------------------------------------------- */

/* 
   FILE: inst.cc
   DESC: This file loads all template code for the various fields  
         and template class in this project and instantiates them. 

   TODO: should only instantiate classes given by the -D option?!?
*/

#include "fields.h"		// finite fields
#include <LiDIA/bigrational.h>	// Qp
#include "charmod.h"		// small prime fields
#include "intmod.h"		// larger prime fields
#include <LiDIA/alg_number.h>
#include "msymbols.h"

// Read in the template function definitions.  
// NOTE: These cannot be compiled and then instantiated at link time!
// They can appear only once in the source code.  They really
// should be in .h files, but that's not the way the previous
// author set up his library.

#include "array.cc"
#include "vector.cc"
#include "matrix.cc"
#include "sparsematrix.cc"
#include "sparse.cc"
#include "lines.cc"
#include "list.cc"

/////////////////////////////////////
// used by many clients.
template class Array<int> ;
template class Vector<int> ;
template ostream& operator<< (ostream&, const Vector<int>&);
template ostream& operator<< (ostream&, const Vector<bigint>&);
template istream& operator>> (istream&, Vector<bigint>&);
template class Array<bigint> ;
template class Vector<bigint> ;
template class Matrix<int> ;
template class Matrix<bigint> ;
template class MatVecList<bigint>;
template class IntVecList<bigint>;

template class Array<MSym>;

template void pol_pseudodiv(Vector<bigrational>& Q, Vector<bigrational>& R,
			   const Vector<bigrational>& A, const Vector<bigrational>& B);
template bigrational pol_resultant(Vector<bigrational> A, Vector<bigrational> B);



///////////////////////////////////////////////////////////////////
// Type F = bigrational. 
template class Array<bigrational>;
template class Vector<bigrational>;
template class VecFac<bigrational>;
template class Matrix<bigrational>;
template class MatVecList<bigrational>;
template class IntVecList<bigrational>;
template class LinesVal<bigrational>;
template class Elem<bigrational>;      
template class SparseMatrix<bigrational>;
template class Sparse<bigrational>;
template ostream& operator<< (ostream&, const Array<bigrational>&);
template ostream& operator<< (ostream&, const Vector<bigrational>&);
template ostream& operator<< (ostream&, const Matrix<bigrational>&);
template istream& operator>>(istream& s, Matrix<bigrational>& v);
template istream& operator>>(istream& s, Vector<bigrational>& v);
template bigrational operator*(const Vector<bigrational>& v,
			       const Vector<bigrational>& w);
template void Eigenvector(Matrix<bigrational>& V,
			 const Matrix<bigrational>& M,
			 const Vector<bigrational>& f);
template void Kernel(Matrix<bigrational>& K,
		     const Matrix<bigrational>& A, double MAX_DEN);
template void Kernel(Matrix<bigrational>& K,
		     const Matrix<bigrational>& A,
		     const Vector<bigrational>& f, double MAX_DEN);
template void Image(Matrix<bigrational>& K,
		     const Matrix<bigrational>& A, double MAX_DEN);
template void Image(Matrix<bigrational>& K,
		     const Matrix<bigrational>& A,
		     const Vector<bigrational>& f, double MAX_DEN);
template bigrational TraceOfProduct(const Matrix<bigrational>& A,
				    const Matrix<bigrational>& B);
template bool operator==(const Matrix<bigrational>&,
			 const Matrix<bigrational>&);

///////////////////////////////////////////////////////////////////
// Type F = ffield_element
				// needed to fool Array::quicksort.
int operator<(const ffield_element&, const ffield_element&) { return 1 ; }
int operator>(const ffield_element&, const ffield_element&) { return 1 ; }
template class Array<ffield_element>;
template class Vector<ffield_element>;
template class VecFac<ffield_element>;
template
ffield_element operator*(const Vector<ffield_element>& v,
			 const Vector<ffield_element>& w);
template class Matrix<ffield_element>;
template class MatVecList<ffield_element>;
template class LinesVal<ffield_element>;
template class Elem<ffield_element>;      
template class SparseMatrix<ffield_element>;
template class Sparse<ffield_element>;
template ostream& operator<< (ostream&, const Array<ffield_element>&);
template ostream& operator<< (ostream&, const Vector<ffield_element>&);
template ostream& operator<< (ostream&, const Matrix<ffield_element>&);
template void Eigenvector(Matrix<ffield_element>& V,
			 const Matrix<ffield_element>& M,
			 const Vector<ffield_element>& f);
template void Kernel(Matrix<ffield_element>& K,
		     const Matrix<ffield_element>& A, double MAX_DEN);
template void Kernel(Matrix<ffield_element>& K,
		     const Matrix<ffield_element>& A,
		     const Vector<ffield_element>& f, double MAX_DEN);
template void Image(Matrix<ffield_element>& K,
		     const Matrix<ffield_element>& A, double MAX_DEN);
template void Image(Matrix<ffield_element>& K,
		     const Matrix<ffield_element>& A,
		     const Vector<ffield_element>& f, double MAX_DEN);
template bool operator==(const Matrix<ffield_element>&,
			 const Matrix<ffield_element>&);



///////////////////////////////////////////////////////////////////
// Type F = charmod
template class Array<charmod>;
template class Vector<charmod>;
template class VecFac<charmod>;
template class Matrix<charmod>;
template class MatVecList<charmod>;
template class LinesVal<charmod>;
template class Elem<charmod>;      
template class SparseMatrix<charmod>;
template class Sparse<charmod>;
template ostream& operator<< (ostream&, const Array<charmod>&);
template ostream& operator<< (ostream&, const Vector<charmod>&);
template ostream& operator<< (ostream&, const Matrix<charmod>&);
template 
charmod operator*(const Vector<charmod>& v, const Vector<charmod>& w);
template void Eigenvector(Matrix<charmod>& V,
			 const Matrix<charmod>& M,
			 const Vector<charmod>& f);
template void Kernel(Matrix<charmod>& K,
		     const Matrix<charmod>& A, double MAX_DEN);
template void Kernel(Matrix<charmod>& K,
		     const Matrix<charmod>& A,
		     const Vector<charmod>& f, double MAX_DEN);
template void Image(Matrix<charmod>& K,
		     const Matrix<charmod>& A, double MAX_DEN);
template void Image(Matrix<charmod>& K,
		     const Matrix<charmod>& A,
		     const Vector<charmod>& f, double MAX_DEN);
template bool operator==(const Matrix<charmod>&,
			 const Matrix<charmod>&);


///////////////////////////////////////////////////////////////////
// Type F = intmod
template class Array<intmod>;
template class Vector<intmod>;
template class VecFac<intmod>;
template class Matrix<intmod>;
template class MatVecList<intmod>;
template class LinesVal<intmod>;
template class Elem<intmod>;      
template class SparseMatrix<intmod>;
template class Sparse<intmod>;
template ostream& operator<< (ostream&, const Array<intmod>&);
template ostream& operator<< (ostream&, const Vector<intmod>&);
template ostream& operator<< (ostream&, const Matrix<intmod>&);
template 
intmod operator*(const Vector<intmod>& v, const Vector<intmod>& w);
template void Eigenvector(Matrix<intmod>& V,
			 const Matrix<intmod>& M,
			 const Vector<intmod>& f);
template void Kernel(Matrix<intmod>& K,
		     const Matrix<intmod>& A, double MAX_DEN);
template void Kernel(Matrix<intmod>& K,
		     const Matrix<intmod>& A,
		     const Vector<intmod>& f, double MAX_DEN);
template void Image(Matrix<intmod>& K,
		     const Matrix<intmod>& A, double MAX_DEN);
template void Image(Matrix<intmod>& K,
		     const Matrix<intmod>& A,
		     const Vector<intmod>& f, double MAX_DEN);
template bool operator==(const Matrix<intmod>&,
			 const Matrix<intmod>&);



///////////////////////////////////////////////////////////////////
// Type F = algnum
template class Array<algnum>;
template class Vector<algnum>;
template class VecFac<algnum>;
template class Matrix<algnum>;
template class MatVecList<algnum>;
template class LinesVal<algnum>;
template class Elem<algnum>;      
template class SparseMatrix<algnum>;
template class Sparse<algnum>;
template ostream& operator<< (ostream&, const Array<algnum>&);
template ostream& operator<< (ostream&, const Vector<algnum>&);
template ostream& operator<< (ostream&, const Matrix<algnum>&);
template 
algnum operator*(const Vector<algnum>& v, const Vector<algnum>& w);
template void Eigenvector(Matrix<algnum>& V,
			 const Matrix<algnum>& M,
			 const Vector<algnum>& f);
template void Kernel(Matrix<algnum>& K,
		     const Matrix<algnum>& A, double MAX_DEN);
template void Kernel(Matrix<algnum>& K,
		     const Matrix<algnum>& A,
		     const Vector<algnum>& f, double MAX_DEN);
template void Image(Matrix<algnum>& K,
		     const Matrix<algnum>& A, double MAX_DEN);
template void Image(Matrix<algnum>& K,
		     const Matrix<algnum>& A,
		     const Vector<algnum>& f, double MAX_DEN);
template bool operator==(const Matrix<algnum>&,
			 const Matrix<algnum>&);




///////////////////////////////////////////////////////////////////
// Type F = bigcomplex
template class Vector<bigcomplex>;
template class Matrix<bigcomplex>;
template ostream& operator<< (ostream&, const Vector<bigcomplex>&);
template ostream& operator<< (ostream&, const Matrix<bigcomplex>&);
template 
bigcomplex operator*(const Vector<bigcomplex>& v, const Vector<bigcomplex>& w);


///////////////////////////// CONVERSIONS //////////////////
template void ChangeField (Matrix<intmod>& A,
			   const Matrix<bigrational>& B) ;







