/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu

    This source file is based on code of John Cremona and Luiz. 
------------------------------------------------------------------------- */

#include "intmod.h"
#include "arith.h"
#include "character.h"

int intmod::pr ;
int intmod::halfpr ;
int intmod::prim_root ;

void
intmod::set_pr (int p)
{
  pr = p ;
  halfpr = pr/2 ;

  // NOTE!! I think this will fail if p is bigger than
  // sqrt(2^31) since then powmod won't work. 

  // compute a primitive root mod p.
  // dumb algorithm, but o.k. since p not that big.
  if (p==2)
    prim_root = 1; 
  else {
    for (prim_root = 2 ; prim_root < p ; prim_root++) {
      int i ;
      for (i = 2 ; i <= (p-1)/2 ; i++)
	if ( ((p-1) % i == 0) && powmod (prim_root, i, p) == 1 ) 
	  i = p ;
      if (i < p) 
	break ;
    }
  }
  if (prim_root == p || powmod (prim_root, p-1, p) != 1) {
    cerr << "Error in intmod::set_pr" << endl ;
    exit(-1) ;
  }
} 

intmod::intmod(const bigrational& x)
{
  bigint a=numerator(x), b=denominator(x);
  if (b%pr==0) {
    cerr << "intmod: Cannot convert " << x << " to a number mod "
	 << pr << "." << endl;
    exit(-1);
  }
  *this=intmod(a)/intmod(b);
}

intmod::intmod(const rootof1& x)
{ 
  *this=x.to_intmod(); 
}
 
void intmod::error(const char* msg) const
{
    cerr << "Error in class intmod:\n" << msg << endl;
    exit(1);
}

intmod inv(const intmod& n) 
{
    int x,y;
    int g = Bezout( n.val, intmod::pr, x, y );
    if( g == 1 ) return intmod(x);
    else n.error("inv called for non-invertible number");
}

ostream& operator<< (ostream& s, const intmod& n)
{
    s << n.val ;
    return s;
}

istream& operator>> (istream& s, intmod& n)
{
    s >> n.val;
    n.val %= intmod::pr;
    return s;
}








