// $Id$

#ifndef _LINES_H
#define _LINES_H

#include "vector.h"
#include "array.h"

// Struct line_info will be used in function garbage_collect
// to store, for each row, the row number and where it starts 
// in array rows.

struct line_info { 
    int row; 
    int start; 
    line_info(int a,int b) {row = a; start = b; }
    line_info(int) {
        // this function exists only so arrays of line_info's 
	// can be created (and initialized to 0). 
    }
    line_info& operator = (const line_info& r1) {
	row = r1.row; start = r1.start;
	return *this;
    }
  bool operator < (const line_info& r) { return start < r.start ; }
  bool operator > (const line_info& r) { return start > r.start ; }
  bool operator == (const line_info& r) {
    return row==r.row && start==r.start; 
  }
  
  line_info() {}
};

template <class T> class Elem {
public:
    int pos;
    T val;
};

class Lines {
protected:
    int nro;
    int size;        // full size of array entries
    int next;        // next available position
    int slack;       // # of zeros put initially after each row 
    long* Nline;     // number of elements in each row
    long* entries;   // single array with all column numbers
    long* Start;     // position in array rows where each row starts
public:
    Lines(long nr);
    Lines( const Lines& );
    ~Lines();

    long& nline( long i ) { return Nline[i]; }
    long& start( long i ) { return Start[i]; }

    inline int entry( int index );     // 0 <= index < size
    inline int entry( int rownumber, int index); // 1 <= rownumber, 0 <= index
    
    long get_size() const { return size; }
    long nrows() const { return nro; }
    inline int available (); 

    void reset_nro( long new_nro );
    void set_slack( long new_slack );
    int get_slack() const { return slack; }
    virtual void resize( int new_size );
    
    void clean();
    void permutation( const Vector<int>& v);
    void take_out(int line, int elem);
    void setrow( int row, int d, long* pos);
    virtual void set(int pos, int value);
//    virtual int put(int line, int value);
    
    void garbage_collect( const Array<line_info>& RI );
    void copy_to_end( int row);
    virtual void error(const char* msg) const;
    void display( int r ) const;
};

template <class T> class  Sparse;

template <class T> class LinesVal: public Lines {
friend class Sparse<T>;
private:
    T* values;
public:
    LinesVal( long nr ) : Lines(nr) { values = 0; }
    LinesVal( const LinesVal& );
    ~LinesVal() { delete [] values; }
    
    int entry( int index ) { return Lines::entry(index); }    
    int entry( int rn, int i) { return Lines::entry(rn,i); }
    void entry( int row, int index, Elem<T>& elem );

    void resize( int new_size );
    void setrow( int row, int d, long* pos, T* val );
    void multiply( int row, const T& alpha);
    
    virtual void error(const char* msg) const;
};

// get entry by index in big array entries.

inline int Lines::entry( int index )
{
    if( index < 0 || index >= size ) 
	error("Out of bounds in function Lines::entry");
    return entries[index];
}

// get entry by row number and index in that row. 
// eg, entry(1,0) returns first element of first row.

inline int Lines::entry( int row, int index)
{
    if( index < 0 || index >= Nline[row] ) 
	error("index out of bound in function Lines::entry");
    return entries[Start[row]+index];
}

inline int Lines::available() {
  /*
    if( entries[next] != 0 ) {
	cout << "**** " << (size-next-1) << "  *****\n";
	return 0;
    }
  */
    return (size-next-1); 
}

#endif
