/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu
------------------------------------------------------------------------- */
// modformulas.h  - Modular formulas

#ifndef MODULARFORMULAS_H
#define MODULARFORMULAS_H
#include "character.h"


class ModularFormulas {
public:
				// Compute the dimension of S_k(N,chi)
				// using Hijikata's trace formula.
  int dimSk (Character& chi, const int& k) ;
				// I don't know how to do this.
  int dimSknew (Character& chi, const int& k)  ;
				// I don't know how to do this.
  int dimMk (const Character& chi, const int& k) const ;
				// returns the index [SL2(Z):Gamma_0(N)]
  int idxG0 (const bigint& N) const ;
				// calculates the number of cusps on X_0(N)
  int cusps0 (const int& N) const ;
				// calculates the genus of X_0(N)
  int genus0 (const int& N) const ;
				// calculates the number of s.s. point
				// at p on X_0(N).
  int ss0 (const int& N, const int& p) const ;
  
				// Compute the dimension of S_k(N)
				// using Shimura's formulas.
  int dimSk (const int& N, const int& k) const ;
  int dimMk (const int& N, const int& k) const ;
				// Compute the dimension of S_k^{new}(N)
				// using Shimura's formulas.
  int dimSknew (const int& N, const int& k) const ;
				// (N PRIME!) calcs. the order
				// of (0)-(\infty) on J_0(N).
  int eisen(const int& N) const ;

				// returns the index [SL2(Z):Gamma_1(N)]
  int idxG1 (const int& N) const ;
				// calculates the number of cusps on X_1(N)
  int cusps1 (const int& N) const ;
				// calculates the genus of X_1(N)
  int genus1 (const int& N) const ;
				// calculates the number of s.s. point
				// at p of X_1(N).
  int ss1 (const int& N, const int& p) const ;  
				// Compute the dimension of S_k(Gamma_1(N))
				// using Shimura's formulas.
  int dimSk1 (const int& N, const int& k) const ;
  int dimMk1 (const int& N, const int& k) const ;
				// Compute the dimension of S_k^{new}(N)
				// using Shimura's formulas.
  int dimSk1new (const int& N, const int& k) const ;
  
  void help()  ;	
  void interactive ()  ;	// simple interactive mode.
private:
  void error (const char* msg) const ;  // error message. 
  void mesg (const char* s) const ;

  // used by hijikata trace formula for dim.
  algnum hij_a (int s) ;
  algnum hij_b (int s) ;
  algnum hij_csfp (int s, int f, int p)  ;
  algnum hij_c (int s, int f) ;
  algnum hij_bc (int s, int f) ;
  algnum hij_abc () ;

  // used by bruce genus stuff
  int mu20 (const int& N) const;
  int mu30 (const int& N) const;
  int mu21 (const int& N) const;
  int mu31 (const int& N) const;
  int mumu (const int& N) const;
  int muXNp (const int& N, const int& p) const;
  int mu2XNp (const int& N, const int& p) const;
  int mu3XNp (const int& N, const int& p) const;
  int cXNp (const int& N, const int& p) const;
  int gXNp (const int& N, const int& p) const;

private:			// used by hijikata's functions.
  int k, N;
  Character* chi;
  rational_factorization fac;
} ;
  
#endif // MODULARFORMULAS_H
