/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu
------------------------------------------------------------------------- */

#ifndef MODULARCALC_H
#define MODULARCALC_H

#include "vector.h"
#include "system.h"
#include "modformulas.h"
#include "h1nchi.h"
#include "display.h"
#include "charpoly.h"
#include "sparse.h"
#include "matrix.h"
#include "tables.h"

// Nice interface to take care of creating the
// right modular symbols calculator.
void StartupModularCalculator(bool expert=false, bool save=true);

template <class F>
class ModularCalculator {
private:
    bool matrix, charpoly, parilog, trace ;
    bool mk, sk, sknew;
    bool plus;
    bool integral;
    bool save;
    Matrix<F> V;
    char Vname[1024];
    Character chi;
    H1NChi<F>* H;
    F foo;
    int N, k;
    char* title;
    
    void status();
    void helpnum(int topic);
    void helpword(char* name);
    void help();
    void init_sk();
    void init_mk();
    void init_V(char c);
    void set_V(int n, const Vector<F>& f);
    void reduce_modp();
    void compute_discriminant();
    void compute_discriminant_new();
    void compute_integral_basis();
    void display_basisfree();
    void display_basiscurrent();
    void cyclic_module();
    void torus_bsdratio();
    void basisheckedual();
    void modular_to_free();
    void manin_to_free();
    void matrix_to_free();
    void torusbsd();
    void toruscinfinity();
    void torusellc4c6();
    void torusellj();
    void toruslseries();
    void torusmoddeg();
    void torusintpairing();
    void torusperiodlattice();
    void torusrealperiod();
    void torustorsion();
    void opcharpoly();
    void opmatrix();
    void optrace();
    void compute_Tn();
    void compute_atkin();
    void act_ZmodNZ();
    void compute_L();
    void compute_winding();
    void compute_conjugation();
    void compute_conjeigen(bool plus);
    void compute_eigen();
    void compute_gen_eigen();
    void compute_aplist();
    void assure_newspace();
    void compute_anlist();
    void basisrational();
    void basisrationalnew();
    void compute_fast_Tp();
    void getheckefactor(int& n, Vector<F>& f);   // n=0 if user cancels.
    void startup();
    
public:
    ModularCalculator (char* _title,
		       const Character& _chi,
		       const int& _k,
		       bool _save=true,
		       bool _plus=true,
		       bool _integral=false) ;
    void interface1();

    // ModularCalculator deletes the H1NChi<F> object, so client code better not!
    ModularCalculator (char* _title, H1NChi<F>* _H) ;
    ~ModularCalculator();
};


#endif // MODULARCALC_H

