/* ----------------------------------------------------------------------
    HECKE: Program for computing with modular forms
    Copyright (C) 1999  William A. Stein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    William A. Stein
    2041 Francisco Street, Apt. 14
    Berkeley, CA  94709
    was@math.berkeley.edu

    This source file is based on code of John Cremona and Luiz. 
------------------------------------------------------------------------- */

// sparsematrix.h

#ifndef _SPARSEMATRIX_H
#define _SPARSEMATRIX_H 

#include "matrix.h"
#include "lines.h"

template <class T> class  Sparse;

template <class T>
class SparseMatrix {
  friend class Sparse<T>;
private:
  LinesVal<T>* rows;
  long nco;
  long nro;
  SparseMatrix(int);
  
public:
  SparseMatrix( long nr = 0, long nc = 0 );
  SparseMatrix( const SparseMatrix& );  
  SparseMatrix( const char* filename );
  
  ~SparseMatrix() { delete rows; }                           
  
  void BuildFromMatrix (const Matrix<T>& M, int num_nonzero_entries = -1) ;
  void BuildFromSparseRep (const Vector<int>& r,
			   const Vector<int>& c,
			   const Vector<T>& val, 
			   int nro=-1, int nco=-1) ;
  const long ncols() const { return nco; }
  const long nrows() const { return nro; }
  const long nrow( int i ) const { return rows->nline(i); }
  long count() const;   // returns number of entries;
  
  void resize_nelem (long nelem) ;

  // note: you *MUST* make sure that you aren't adding
  // more rows to the matrix than you specified
  // in call to ::resize_nelem !!  (I think.)
  void my_setrow (long r, int d, long* posi, T* val) ;
  
  void as_Matrix( Matrix<T>& ) const; 
  T operator() (long i, long j) const;
  SparseMatrix& operator= (const SparseMatrix&);
  
  void row(int r, Vector<T> vec);    // 1 <= r <= nro
  
  bool equal( const SparseMatrix<T>& ) const;
  void display() const;
  virtual void error(const char* msg) const;
  void normalize(int row, int col);    
  friend bool operator == (const SparseMatrix<T>&, const SparseMatrix<T>& );
  friend bool operator != (const SparseMatrix<T>&, const SparseMatrix<T>& );
  Matrix<T> SparseMatrix<T>::operator * ( const Matrix<T>& );
};

template <class T> inline
bool operator == (const SparseMatrix<T>& sm1, const SparseMatrix<T>& sm2)
{
  return sm1.equal( sm2 );
}

template <class T> inline
bool operator != (const SparseMatrix<T>& sm1, const SparseMatrix<T>& sm2)
{
    return !sm1.equal( sm2 );
}

#endif



