// system.h -- system dependent/related functions, IO wrappers. 

#ifndef SYSTEM_H
#define SYSTEM_H

const int MIN_INT=-2147483647;
const int MAX_INT=2147483647;

typedef long long longlong;	
//const longlong MAX_LONGLONG=9223372036854775807;
//const longlong MIN_LONGLONG=9223372036854775807;


// IO FUNCTIONS
#include <iostream.h>
#include <fstream.h>

void eof_check();

// Returns an int, as entered from input stream in (default = cin).
// prompt = will be displayed, if nonzero.
// min = min allowable input
// max = max allowable input.
// If n < min or n > max, display a message like:
//      Please enter an integer between -3 and 10: 
// and ask again for input. 
// terminate_on_eof = true  -- call exit(-1) if can't read (eof, etc)
//                  = false -- make n=0 if can't read
// * character input, e.g., "x", is treated as a zero. 
int get_int(const char* prompt=NULL,
	    int min=MIN_INT,
	    int max=MAX_INT,
	    istream& in=cin,
	    bool terminate_on_eof=true);

bool get_bool(const char* prompt=NULL,
	      bool def=true,
	      istream& in=cin,
	      bool terminate_on_eof=true);

char get_char(const char* prompt=NULL,
	      istream& in=cin,
	      bool terminate_on_eof=true);

void get_str(char* str,
	     int maxlen,
	     const char* prompt=NULL,
	     istream& in=cin,
	     bool terminate_on_eof=true);

char* data_write (const char* fname, int N);
char* data_read  (const char* fname, int N);
char* userdata_write (const char* fname);
char* userdata_read  (const char* fname);

// kill program.
void die(const char* msg=NULL);

#include <sys/times.h>
#include <sys/types.h>
#include <sys/param.h>

// returns time in seconds since last call to reset_timer.
double get_time () ;  
void reset_timer() ;

// returns a pointer to something like "Sat Apr 10 14:03:18 PDT 1999".
char* date_stamp() ;

// returns a pointer to a string like 
// "William Stein (was@math.berkeley.edu)"
char* signature_stamp() ;

// **************** Messages class ********************
class Mesg {
  bool show;
  bool file;
  ofstream of;      // any output stream should be allowed
public:
  Mesg() : show(true), file(false) {}
  ~Mesg() {}
  void on() { show=true; } 
  void off() {show=false; }
  bool is_on() { return show; }
  void mesg(const float f);
  void mesg(const char* str);
  void mesg(const char* str, const int n);
  void mesg(const int n, const char* str);
} ;

static Mesg Messages;

inline bool mesg_on() { return Messages.is_on(); }
inline void mesg(const char* str) { Messages.mesg(str); }
inline void mesg(const char* str, const char* str2) { Messages.mesg(str); Messages.mesg(str2); }
inline void mesg(const char* str, const int n) { Messages.mesg(str,n); }
inline void mesg(const int n, const char* str) { Messages.mesg(n,str); }
inline void mesg(const float f) { Messages.mesg(f); }
inline void mesgln(const char* str) { Messages.mesg(str); Messages.mesg("\n"); }
inline void mesgln(const char* str, const int n) { Messages.mesg(str,n); 
   Messages.mesg("\n");}
inline void mesgln(const int n, const char* str) { Messages.mesg(n,str); 
   Messages.mesg("\n");}
inline void mesgln(const float f) { Messages.mesg(f); 
   Messages.mesg("\n");}


void stub(const char* funcname);

char* username();
  
#endif // SYSTEM_H





